@@ -39,6 +39,7 @@
 #define	DBG_MASK	0x80
 #define	MAX_HEX_LINES	64000
 #define HAVE_OCTASIC	1
+#define DEF_SPAN_SPEC_FORMAT	"*:%c1" /* %c: 'E' or 'T' */
 
 static char	*progname;
 
@@ -50,6 +51,7 @@ static void usage()
 #if HAVE_OCTASIC
 	fprintf(stderr, "\t\t[-O]               # Load Octasic firmware\n");
 	fprintf(stderr, "\t\t[-o]               # Show Octasic version\n");
+	fprintf(stderr, "\t\t[-S <pri-spec>]    # Set PRI type specification string\n");
 #endif
 	fprintf(stderr, "\t\t[-F]               # Load FPGA firmware\n");
 	fprintf(stderr, "\t\t[-p]               # Load PIC firmware\n");
@@ -164,11 +166,13 @@ int main(int argc, char *argv[])
 	int			opt_ecver = 0;
 #if HAVE_OCTASIC
 	int			opt_alaw = 0;
+	const char		*span_spec = NULL;
+	char			def_span_spec[sizeof(DEF_SPAN_SPEC_FORMAT)];
 #endif
 	int			opt_dest = 0;
 	int			opt_sum = 0;
 	enum dev_dest		dest = DEST_NONE;
-	const char		options[] = "vd:D:EFOopA";
+	const char		options[] = "vd:D:EFOopAS:";
 	int			iface_num;
 	int			ret;
 
@@ -210,6 +214,9 @@ int main(int argc, char *argv[])
 			case 'A':
 				opt_alaw = 1;
 				break;
+			case 'S':
+				span_spec = optarg;
+				break;
 #endif
 			case 'p':
 				opt_pic = 1;
@@ -247,6 +254,13 @@ int main(int argc, char *argv[])
 		ERR("Missing device path.\n");
 		usage();
 	}
+# ifdef HAVE_OCTASIC
+	if (!span_spec) {
+		snprintf(def_span_spec, sizeof(def_span_spec),
+				DEF_SPAN_SPEC_FORMAT, opt_alaw? 'E' : 'T');
+		span_spec = def_span_spec;
+	}
+#endif
 	if(opt_dest) {
 		/*
 		 * MPP Interface
@@ -290,7 +304,7 @@ int main(int argc, char *argv[])
 			}
 #if HAVE_OCTASIC
 		} else if (opt_echo) {
-			if((ret = load_echo(astribank, argv[optind], opt_alaw)) < 0) {
+			if((ret = load_echo(astribank, argv[optind], opt_alaw, span_spec)) < 0) {
 				ERR("%s: Loading ECHO's failed\n", devpath);
 				return 1;
 			}
