@@ -5,7 +5,7 @@ package Dahdi::Xpp::Xpd;
 # This program is free software; you can redistribute and/or
 # modify it under the same terms as Perl itself.
 #
-# $Id: Xpd.pm 10032 2011-07-10 16:25:18Z tzafrir $
+# $Id$
 #
 use strict;
 use Dahdi::Utils;
@@ -52,11 +52,6 @@ Applicable only to digital (BRI/PRI) modules and always 0 for others.
 
 Textual name: E.g. C<XPD-10>.
 
-=head1 dir
-
-The ProcFS directory with information about the XPD. e.g.
-C</proc/xpp/XBUS-00/XPD-10>.
-
 =head1 sysfs_dir
 
 The SysFS directory with information about the module. E.g.
@@ -104,73 +99,21 @@ my %file_warned;	# Prevent duplicate warnings about same file.
 
 sub xpd_attr_path($@) {
 	my $self = shift || die;
+	my $xbus = $self->xbus;
 	my ($busnum, $unitnum, $subunitnum, @attr) = (
-		$self->xbus->num,
+		$xbus->num,
 		$self->unit,
 		$self->subunit,
 		@_);
 	foreach my $attr (@attr) {
-		my $file = sprintf "$Dahdi::Xpp::sysfs_xpds/%02d:%1d:%1d/$attr",
-		   $busnum, $unitnum, $subunitnum;
-		unless(-f $file) {
-			my $procfile = sprintf "/proc/xpp/XBUS-%02d/XPD-%1d%1d/$attr",
-			   $busnum, $unitnum, $subunitnum;
-			warn "$0: warning - OLD DRIVER: missing '$file'. Fall back to /proc\n"
-				unless $file_warned{$attr}++;
-			$file = $procfile;
-		}
+		my $file = sprintf "%s/%02d:%1d:%1d/$attr",
+		   $xbus->sysfs_dir, $busnum, $unitnum, $subunitnum;
 		next unless -f $file;
 		return $file;
 	}
 	return undef;
 }
 
-# Backward compat plug for old /proc interface...
-sub xpd_old_gettype($) {
-	my $xpd = shift || die;
-	my $summary = "/proc/xpp/" . $xpd->fqn . "/summary";
-	open(F, $summary) or die "Failed to open '$summary': $!";
-	my $head = <F>;
-	close F;
-	chomp $head;
-	$head =~ s/^XPD-\d+\s+\(//;
-	$head =~ s/,.*//;
-	return $head;
-}
-
-sub xpd_old_getspan($) {
-	my $xpd = shift || die;
-	my $dahdi_registration = "/proc/xpp/" . $xpd->fqn . "/dahdi_registration";
-	open(F, $dahdi_registration) or die "Failed to open '$dahdi_registration': $!";
-	my $head = <F>;
-	close F;
-	chomp $head;
-	return $head;
-}
-
-sub xpd_old_getoffhook($) {
-	my $xpd = shift || die;
-	my $summary = "/proc/xpp/" . $xpd->fqn . "/summary";
-	my $channels;
-
-	local $/ = "\n";
-	open(F, "$summary") || die "Failed opening $summary: $!\n";
-	my $head = <F>;
-	chomp $head;	# "XPD-00 (BRI_TE ,card present, span 3)"
-	my $offhook;
-	while(<F>) {
-		chomp;
-		if(s/^\s*offhook\s*:\s*//) {
-			s/\s*$//;
-			$offhook = $_;
-			$offhook || die "No channels in '$summary'";
-			last;
-		}
-	}
-	close F;
-	return $offhook;
-}
-
 my %attr_missing_warned;	# Prevent duplicate warnings
 
 sub xpd_driver_getattr($$) {
@@ -198,10 +141,6 @@ sub xpd_getattr($$) {
 	$attr = lc($attr);
 	my $file = $xpd->xpd_attr_path(lc($attr));
 
-	# Handle special cases for backward compat
-	return xpd_old_gettype($xpd) if $attr eq 'type' and !defined $file;
-	return xpd_old_getspan($xpd) if $attr eq 'span' and !defined $file;
-	return xpd_old_getoffhook($xpd) if $attr eq 'offhook' and !defined $file;
 	if(!defined($file)) {
 		warn "$0: xpd_getattr($attr) -- Missing attribute.\n" if
 			$attr_missing_warned{$attr};
@@ -279,20 +218,18 @@ sub xpds_by_spanno() {
 	return @idx;
 }
 
-sub new($$$$$) {
+sub new($$$) {
 	my $pack = shift or die "Wasn't called as a class method\n";
-	my $xbus = shift || die;
-	my $unit = shift;	# May be zero
-	my $subunit = shift;	# May be zero
-	my $procdir = shift || die;
-	my $sysfsdir = shift || die;
+	my $xbus = shift or die;
+	my $xpdstr = shift or die;
+	my $sysfsdir = sprintf "%s/%s", $xbus->sysfs_dir, $xpdstr;
+	my ($busnum, $unit, $subunit) = split(/:/, $xpdstr);
 	my $self = {
 		XBUS		=> $xbus,
 		ID		=> sprintf("%1d%1d", $unit, $subunit),
 		FQN		=> $xbus->name . "/" . "XPD-$unit$subunit",
 		UNIT		=> $unit,
 		SUBUNIT		=> $subunit,
-		DIR		=> $procdir,
 		SYSFS_DIR	=> $sysfsdir,
 		};
 	bless $self, $pack;
@@ -319,7 +256,7 @@ sub new($$$$$) {
 		}
 	}
 	$self->{IS_DIGITAL} = ( $self->{IS_BRI} || $self->{IS_PRI} );
-	Dahdi::Xpp::Line->create_all($self, $procdir);
+	Dahdi::Xpp::Line->create_all($self);
 	return $self;
 }
 
